#### A Kuramoto Model of Fad Generation####

##Created for the 2023 ISDC in Chicago July 23 - 27 2023

## James Paine ##
## j.paine@bucknell.edu ##
## http://jpaine.info ##

## Note:  This code was designed to be run in the RStudio IDE. Running in base-R
##        or another IDE may cause portions of this code involving graphics to
##        fail or produce unexpected outputs.

#### Environment Setup ####

#Set the working directory to the folder this script is located in
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

#### SIMULATION PARAMETERS AND INPUTS ####

#Simulation time horizon and Euler integration step size
Horizon = 100
StepSize = 0.0625

#Number of oscillators
N = 10

#Coupling Strengths
K = 2

#Setting Natural Frequencies
Shared_Omega = TRUE
  Global_Omega = .2
Omega = c() #If Shared_Omega is False, insert a vector of omegas here

#Add noise to the natural frequencies
Normal_Omega = FALSE
  Normal_Omega_Mean = 5
  Normal_Omega_SD = 1

#Add delays and noise to the perception of distances
Perception_Delay = 0
Perception_Noise = TRUE
  Perception_Noise_Bias = 0
  Perception_Noise_SD = pi/2
  Noise_Seed = 123

#Control the starting position of the oscillators
Equidistant_Start = TRUE #Place all the oscillators equally around the unit circle
Clustered_Start = FALSE  #Cluster the oscillators together in the same starting spot
  Starting_Theta = 0

#Output controls
PlotResults = TRUE
SuppressUntilEnd = FALSE


#### Initial Vector and Dataframe Setup ####

t_range = seq(from = 0, to = Horizon, by = StepSize )
StepCount = length(t_range)

dtheta = data.frame(matrix(NA,nrow = StepCount,ncol=N))
rownames(dtheta) = t_range
colnames(dtheta) = (1:N)


theta = data.frame(matrix(NA,nrow = StepCount,ncol=N))
rownames(theta) = t_range
colnames(theta) = (1:N)


if (Perception_Noise == TRUE) {
  set.seed(Noise_Seed)
}

if (Shared_Omega == TRUE) {
  Omega = c()
  Omega[1:N] = Global_Omega
}

if (Normal_Omega == TRUE){
  Omega = rnorm(N,mean = Normal_Omega_Mean, sd = Normal_Omega_SD)
}


if (Equidistant_Start == TRUE) {
  for (i in (1:N)) {
    theta[1,i] = (2*pi/N)*(i-1)
  }
}

if (Clustered_Start == TRUE) {
  theta[1,] = Starting_Theta
}

#Set up plotting outputs
while (!is.null(dev.list()))  dev.off()
graphics.off()

if ((PlotResults == TRUE)) {
  
  PlotFrames = list()
  
  # Create new output window
  if (SuppressUntilEnd == TRUE) {
    
    #Reset the plotting window
    while (!is.null(dev.list()))  dev.off()
    print(plot(1))
    
  } else {
    
    dev.new(width=6, height=6, noRStudioGD = TRUE, units  = "in") 
    # Adjust the margin of plot (bottom,left,top,right)
    par(mar=c(4, 5, 2, 1))
    
  }
  
  NumPoints = 100
  
  CircleThetaSweep = seq(from = 0, to = 2*pi, by = 2*pi/NumPoints)
  
  #Draw unit circle
  CircleX = cos(CircleThetaSweep)
  CircleY = sin(CircleThetaSweep)
  
  plot(CircleX,CircleY,
       xlab="",ylab="",
       xlim=c(-1.1,1.1),ylim=c(-1.1,1.1),
       type='l',lwd=1, lty = 1)
  

}


#### Main Loop over Time ####

FrameCount = 0
for (step in (1:StepCount)) {
  
  if (step>1) {
    theta[step,] = theta[step-1,]
  }
  
  for (i in (1:N)) {
    #Determine the perceived distances
    distances = c()
    
    Current_Time = t_range[step]
    PrecievedTime = max(0,Current_Time - Perception_Delay)
    PrecievedStep = which.min(abs(t_range - PrecievedTime))
    
    for (j in (1:N)) {
      distances[j] = (theta[PrecievedStep,j] - theta[PrecievedStep,i])
    }
    
    
    if (Perception_Noise == TRUE) {
      distances = distances + rnorm(N,Perception_Noise_Bias,Perception_Noise_SD)
    }
    
    #Determine Effect
    dtheta[step,i]=Omega[i]+(K/N)*sum(sin(distances))
  }
  
  theta[step,] = theta[step,] + dtheta[step,]*StepSize
  
  if ((PlotResults == TRUE)) {

    #dev.flush()
    PlotX = cos(theta[step,])
    PlotY = sin(theta[step,])
    
    NumPoints = 100
    
    CircleThetaSweep = seq(from = 0, to = 2*pi, by = 2*pi/NumPoints)
    
    #Draw unit circle
    CircleX = cos(CircleThetaSweep)
    CircleY = sin(CircleThetaSweep)

    #Redraw cicle
    plot(CircleX,CircleY,
         xlab="Dimension 1",ylab="Dimension 2",
         xlim=c(-1.1,1.1),ylim=c(-1.1,1.1),
         type='l',lwd=1, lty = 1)
    
    #add dots
    points(PlotX,PlotY, 
           cex = 5, lwd = 2,
           pch = 21, col = 'black', bg = 'lightgrey')
    #inidicate the timestep
    mtext(paste("T =",floor(t_range[step])), 3, adj=0, line=1)
    
    #Save the graphic for later export of viewing
    FrameCount = FrameCount+1
    PlotFrames[[FrameCount]] = recordPlot()
    
  } #End plotting
  
} #next step

#Draw a single figure showing the evolution of postions over time
# Note: this graph can be very misleading.
if (PlotResults == FALSE) {
  dev.new(width=6, height=3.75, noRStudioGD = TRUE, units  = "in") 
  matplot(t_range,theta, type = "l", lwd = "3",xlab = "Time",ylab="Angular Position (radians)")
}

#### OPTIONAL: View Specific Plots ####
SinglePlots = FALSE  

if (SinglePlots == TRUE) {
  
  #Create a new output window
  dev.new(width=6, height=6, noRStudioGD = TRUE, units  = "in") 
  # Adjust the margin of plot (bottom,left,top,right)
  par(mar=c(4, 5, 2, 1))
  
  #This loop replays the entire sequence
  for (frame in (1:max(FrameCount))) {
    replayPlot(PlotFrames[[frame]])
  }
  
  #Create a new output window
  dev.new(width=6, height=6, noRStudioGD = TRUE, units  = "in") 
  # Adjust the margin of plot (bottom,left,top,right)
  par(mar=c(4, 5, 2, 1))
  
  #This code plots the graph at a specific time
  PlotTime = 5
  
  #Find closest time
  PlotStep = which.min(abs(t_range - PlotTime))
  replayPlot(PlotFrames[[PlotStep]])
  

}

